from azure_lmdc.azure_wrapper_class import Azure_wrapper


def main():
    azure_client = Azure_wrapper()
    print("Atualice os credencias do Azure SDK no Environment ..")


if __name__=="__main__":
    main()